var NodeType$1;
(function (NodeType) {
    NodeType[NodeType["Document"] = 0] = "Document";
    NodeType[NodeType["DocumentType"] = 1] = "DocumentType";
    NodeType[NodeType["Element"] = 2] = "Element";
    NodeType[NodeType["Text"] = 3] = "Text";
    NodeType[NodeType["CDATA"] = 4] = "CDATA";
    NodeType[NodeType["Comment"] = 5] = "Comment";
})(NodeType$1 || (NodeType$1 = {}));

function parseCSSText(cssText) {
    const res = {};
    const listDelimiter = /;(?![^(]*\))/g;
    const propertyDelimiter = /:(.+)/;
    const comment = /\/\*.*?\*\//g;
    cssText
        .replace(comment, '')
        .split(listDelimiter)
        .forEach(function (item) {
        if (item) {
            const tmp = item.split(propertyDelimiter);
            tmp.length > 1 && (res[camelize(tmp[0].trim())] = tmp[1].trim());
        }
    });
    return res;
}
function toCSSText(style) {
    const properties = [];
    for (const name in style) {
        const value = style[name];
        if (typeof value !== 'string')
            continue;
        const normalizedName = hyphenate(name);
        properties.push(`${normalizedName}: ${value};`);
    }
    return properties.join(' ');
}
const camelizeRE = /-([a-z])/g;
const CUSTOM_PROPERTY_REGEX = /^--[a-zA-Z0-9-]+$/;
const camelize = (str) => {
    if (CUSTOM_PROPERTY_REGEX.test(str))
        return str;
    return str.replace(camelizeRE, (_, c) => (c ? c.toUpperCase() : ''));
};
const hyphenateRE = /\B([A-Z])/g;
const hyphenate = (str) => {
    return str.replace(hyphenateRE, '-$1').toLowerCase();
};

class BaseRRNode {
    constructor(...args) {
        this.childNodes = [];
        this.parentElement = null;
        this.parentNode = null;
        this.ELEMENT_NODE = NodeType.ELEMENT_NODE;
        this.TEXT_NODE = NodeType.TEXT_NODE;
    }
    get firstChild() {
        return this.childNodes[0] || null;
    }
    get lastChild() {
        return this.childNodes[this.childNodes.length - 1] || null;
    }
    get nextSibling() {
        const parentNode = this.parentNode;
        if (!parentNode)
            return null;
        const siblings = parentNode.childNodes;
        const index = siblings.indexOf(this);
        return siblings[index + 1] || null;
    }
    contains(node) {
        if (node === this)
            return true;
        for (const child of this.childNodes) {
            if (child.contains(node))
                return true;
        }
        return false;
    }
    appendChild(_newChild) {
        throw new Error(`RRDomException: Failed to execute 'appendChild' on 'RRNode': This RRNode type does not support this method.`);
    }
    insertBefore(_newChild, _refChild) {
        throw new Error(`RRDomException: Failed to execute 'insertBefore' on 'RRNode': This RRNode type does not support this method.`);
    }
    removeChild(node) {
        throw new Error(`RRDomException: Failed to execute 'removeChild' on 'RRNode': This RRNode type does not support this method.`);
    }
    toString() {
        return 'RRNode';
    }
}
function BaseRRDocumentImpl(RRNodeClass) {
    return class BaseRRDocument extends RRNodeClass {
        constructor() {
            super(...arguments);
            this.nodeType = NodeType.DOCUMENT_NODE;
            this.nodeName = '#document';
            this.compatMode = 'CSS1Compat';
            this.RRNodeType = NodeType$1.Document;
            this.textContent = null;
        }
        get documentElement() {
            return (this.childNodes.find((node) => node.RRNodeType === NodeType$1.Element &&
                node.tagName === 'HTML') || null);
        }
        get body() {
            var _a;
            return (((_a = this.documentElement) === null || _a === void 0 ? void 0 : _a.childNodes.find((node) => node.RRNodeType === NodeType$1.Element &&
                node.tagName === 'BODY')) || null);
        }
        get head() {
            var _a;
            return (((_a = this.documentElement) === null || _a === void 0 ? void 0 : _a.childNodes.find((node) => node.RRNodeType === NodeType$1.Element &&
                node.tagName === 'HEAD')) || null);
        }
        get implementation() {
            return this;
        }
        get firstElementChild() {
            return this.documentElement;
        }
        appendChild(childNode) {
            const nodeType = childNode.RRNodeType;
            if (nodeType === NodeType$1.Element ||
                nodeType === NodeType$1.DocumentType) {
                if (this.childNodes.some((s) => s.RRNodeType === nodeType)) {
                    throw new Error(`RRDomException: Failed to execute 'appendChild' on 'RRNode': Only one ${nodeType === NodeType$1.Element ? 'RRElement' : 'RRDoctype'} on RRDocument allowed.`);
                }
            }
            childNode.parentElement = null;
            childNode.parentNode = this;
            this.childNodes.push(childNode);
            return childNode;
        }
        insertBefore(newChild, refChild) {
            const nodeType = newChild.RRNodeType;
            if (nodeType === NodeType$1.Element ||
                nodeType === NodeType$1.DocumentType) {
                if (this.childNodes.some((s) => s.RRNodeType === nodeType)) {
                    throw new Error(`RRDomException: Failed to execute 'insertBefore' on 'RRNode': Only one ${nodeType === NodeType$1.Element ? 'RRElement' : 'RRDoctype'} on RRDocument allowed.`);
                }
            }
            if (refChild === null)
                return this.appendChild(newChild);
            const childIndex = this.childNodes.indexOf(refChild);
            if (childIndex == -1)
                throw new Error("Failed to execute 'insertBefore' on 'RRNode': The RRNode before which the new node is to be inserted is not a child of this RRNode.");
            this.childNodes.splice(childIndex, 0, newChild);
            newChild.parentElement = null;
            newChild.parentNode = this;
            return newChild;
        }
        removeChild(node) {
            const indexOfChild = this.childNodes.indexOf(node);
            if (indexOfChild === -1)
                throw new Error("Failed to execute 'removeChild' on 'RRDocument': The RRNode to be removed is not a child of this RRNode.");
            this.childNodes.splice(indexOfChild, 1);
            node.parentElement = null;
            node.parentNode = null;
            return node;
        }
        open() {
            this.childNodes = [];
        }
        close() {
        }
        write(content) {
            let publicId;
            if (content ===
                '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "">')
                publicId = '-//W3C//DTD XHTML 1.0 Transitional//EN';
            else if (content ===
                '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN" "">')
                publicId = '-//W3C//DTD HTML 4.0 Transitional//EN';
            if (publicId) {
                const doctype = this.createDocumentType('html', publicId, '');
                this.open();
                this.appendChild(doctype);
            }
        }
        createDocument(_namespace, _qualifiedName, _doctype) {
            return new BaseRRDocument();
        }
        createDocumentType(qualifiedName, publicId, systemId) {
            const doctype = new (BaseRRDocumentTypeImpl(BaseRRNode))(qualifiedName, publicId, systemId);
            doctype.ownerDocument = this;
            return doctype;
        }
        createElement(tagName) {
            const element = new (BaseRRElementImpl(BaseRRNode))(tagName);
            element.ownerDocument = this;
            return element;
        }
        createElementNS(_namespaceURI, qualifiedName) {
            return this.createElement(qualifiedName);
        }
        createTextNode(data) {
            const text = new (BaseRRTextImpl(BaseRRNode))(data);
            text.ownerDocument = this;
            return text;
        }
        createComment(data) {
            const comment = new (BaseRRCommentImpl(BaseRRNode))(data);
            comment.ownerDocument = this;
            return comment;
        }
        createCDATASection(data) {
            const CDATASection = new (BaseRRCDATASectionImpl(BaseRRNode))(data);
            CDATASection.ownerDocument = this;
            return CDATASection;
        }
        toString() {
            return 'RRDocument';
        }
    };
}
function BaseRRDocumentTypeImpl(RRNodeClass) {
    return class BaseRRDocumentType extends RRNodeClass {
        constructor(qualifiedName, publicId, systemId) {
            super();
            this.nodeType = NodeType.DOCUMENT_TYPE_NODE;
            this.RRNodeType = NodeType$1.DocumentType;
            this.textContent = null;
            this.name = qualifiedName;
            this.publicId = publicId;
            this.systemId = systemId;
            this.nodeName = qualifiedName;
        }
        toString() {
            return 'RRDocumentType';
        }
    };
}
function BaseRRElementImpl(RRNodeClass) {
    return class BaseRRElement extends RRNodeClass {
        constructor(tagName) {
            super();
            this.nodeType = NodeType.ELEMENT_NODE;
            this.RRNodeType = NodeType$1.Element;
            this.attributes = {};
            this.shadowRoot = null;
            this.tagName = tagName.toUpperCase();
            this.nodeName = tagName.toUpperCase();
        }
        get textContent() {
            let result = '';
            this.childNodes.forEach((node) => (result += node.textContent));
            return result;
        }
        set textContent(textContent) {
            this.childNodes = [this.ownerDocument.createTextNode(textContent)];
        }
        get classList() {
            return new ClassList(this.attributes.class, (newClassName) => {
                this.attributes.class = newClassName;
            });
        }
        get id() {
            return this.attributes.id || '';
        }
        get className() {
            return this.attributes.class || '';
        }
        get style() {
            const style = (this.attributes.style
                ? parseCSSText(this.attributes.style)
                : {});
            const hyphenateRE = /\B([A-Z])/g;
            style.setProperty = (name, value, priority) => {
                if (hyphenateRE.test(name))
                    return;
                const normalizedName = camelize(name);
                if (!value)
                    delete style[normalizedName];
                else
                    style[normalizedName] = value;
                if (priority === 'important')
                    style[normalizedName] += ' !important';
                this.attributes.style = toCSSText(style);
            };
            style.removeProperty = (name) => {
                if (hyphenateRE.test(name))
                    return '';
                const normalizedName = camelize(name);
                const value = style[normalizedName] || '';
                delete style[normalizedName];
                this.attributes.style = toCSSText(style);
                return value;
            };
            return style;
        }
        getAttribute(name) {
            return this.attributes[name] || null;
        }
        setAttribute(name, attribute) {
            this.attributes[name] = attribute;
        }
        setAttributeNS(_namespace, qualifiedName, value) {
            this.setAttribute(qualifiedName, value);
        }
        removeAttribute(name) {
            delete this.attributes[name];
        }
        appendChild(newChild) {
            this.childNodes.push(newChild);
            newChild.parentNode = this;
            newChild.parentElement = this;
            return newChild;
        }
        insertBefore(newChild, refChild) {
            if (refChild === null)
                return this.appendChild(newChild);
            const childIndex = this.childNodes.indexOf(refChild);
            if (childIndex == -1)
                throw new Error("Failed to execute 'insertBefore' on 'RRNode': The RRNode before which the new node is to be inserted is not a child of this RRNode.");
            this.childNodes.splice(childIndex, 0, newChild);
            newChild.parentElement = this;
            newChild.parentNode = this;
            return newChild;
        }
        removeChild(node) {
            const indexOfChild = this.childNodes.indexOf(node);
            if (indexOfChild === -1)
                throw new Error("Failed to execute 'removeChild' on 'RRElement': The RRNode to be removed is not a child of this RRNode.");
            this.childNodes.splice(indexOfChild, 1);
            node.parentElement = null;
            node.parentNode = null;
            return node;
        }
        attachShadow(_init) {
            const shadowRoot = this.ownerDocument.createElement('SHADOWROOT');
            this.shadowRoot = shadowRoot;
            return shadowRoot;
        }
        dispatchEvent(_event) {
            return true;
        }
        toString() {
            let attributeString = '';
            for (const attribute in this.attributes) {
                attributeString += `${attribute}="${this.attributes[attribute]}" `;
            }
            return `${this.tagName} ${attributeString}`;
        }
    };
}
function BaseRRMediaElementImpl(RRElementClass) {
    return class BaseRRMediaElement extends RRElementClass {
        attachShadow(_init) {
            throw new Error(`RRDomException: Failed to execute 'attachShadow' on 'RRElement': This RRElement does not support attachShadow`);
        }
        play() {
            this.paused = false;
        }
        pause() {
            this.paused = true;
        }
    };
}
function BaseRRTextImpl(RRNodeClass) {
    return class BaseRRText extends RRNodeClass {
        constructor(data) {
            super();
            this.nodeType = NodeType.TEXT_NODE;
            this.nodeName = '#text';
            this.RRNodeType = NodeType$1.Text;
            this.data = data;
        }
        get textContent() {
            return this.data;
        }
        set textContent(textContent) {
            this.data = textContent;
        }
        toString() {
            return `RRText text=${JSON.stringify(this.data)}`;
        }
    };
}
function BaseRRCommentImpl(RRNodeClass) {
    return class BaseRRComment extends RRNodeClass {
        constructor(data) {
            super();
            this.nodeType = NodeType.COMMENT_NODE;
            this.nodeName = '#comment';
            this.RRNodeType = NodeType$1.Comment;
            this.data = data;
        }
        get textContent() {
            return this.data;
        }
        set textContent(textContent) {
            this.data = textContent;
        }
        toString() {
            return `RRComment text=${JSON.stringify(this.data)}`;
        }
    };
}
function BaseRRCDATASectionImpl(RRNodeClass) {
    return class BaseRRCDATASection extends RRNodeClass {
        constructor(data) {
            super();
            this.nodeName = '#cdata-section';
            this.nodeType = NodeType.CDATA_SECTION_NODE;
            this.RRNodeType = NodeType$1.CDATA;
            this.data = data;
        }
        get textContent() {
            return this.data;
        }
        set textContent(textContent) {
            this.data = textContent;
        }
        toString() {
            return `RRCDATASection data=${JSON.stringify(this.data)}`;
        }
    };
}
class ClassList {
    constructor(classText, onChange) {
        this.classes = [];
        this.add = (...classNames) => {
            for (const item of classNames) {
                const className = String(item);
                if (this.classes.indexOf(className) >= 0)
                    continue;
                this.classes.push(className);
            }
            this.onChange && this.onChange(this.classes.join(' '));
        };
        this.remove = (...classNames) => {
            this.classes = this.classes.filter((item) => classNames.indexOf(item) === -1);
            this.onChange && this.onChange(this.classes.join(' '));
        };
        if (classText) {
            const classes = classText.trim().split(/\s+/);
            this.classes.push(...classes);
        }
        this.onChange = onChange;
    }
}
var NodeType;
(function (NodeType) {
    NodeType[NodeType["PLACEHOLDER"] = 0] = "PLACEHOLDER";
    NodeType[NodeType["ELEMENT_NODE"] = 1] = "ELEMENT_NODE";
    NodeType[NodeType["ATTRIBUTE_NODE"] = 2] = "ATTRIBUTE_NODE";
    NodeType[NodeType["TEXT_NODE"] = 3] = "TEXT_NODE";
    NodeType[NodeType["CDATA_SECTION_NODE"] = 4] = "CDATA_SECTION_NODE";
    NodeType[NodeType["ENTITY_REFERENCE_NODE"] = 5] = "ENTITY_REFERENCE_NODE";
    NodeType[NodeType["ENTITY_NODE"] = 6] = "ENTITY_NODE";
    NodeType[NodeType["PROCESSING_INSTRUCTION_NODE"] = 7] = "PROCESSING_INSTRUCTION_NODE";
    NodeType[NodeType["COMMENT_NODE"] = 8] = "COMMENT_NODE";
    NodeType[NodeType["DOCUMENT_NODE"] = 9] = "DOCUMENT_NODE";
    NodeType[NodeType["DOCUMENT_TYPE_NODE"] = 10] = "DOCUMENT_TYPE_NODE";
    NodeType[NodeType["DOCUMENT_FRAGMENT_NODE"] = 11] = "DOCUMENT_FRAGMENT_NODE";
})(NodeType || (NodeType = {}));

const nwsapi = require('nwsapi');
const cssom = require('cssom');
const cssstyle = require('cssstyle');
class RRNode extends BaseRRNode {
}
class RRWindow {
    constructor() {
        this.scrollLeft = 0;
        this.scrollTop = 0;
    }
    scrollTo(options) {
        if (!options)
            return;
        if (typeof options.left === 'number')
            this.scrollLeft = options.left;
        if (typeof options.top === 'number')
            this.scrollTop = options.top;
    }
}
class RRDocument extends BaseRRDocumentImpl(RRNode) {
    constructor() {
        super(...arguments);
        this.nodeName = '#document';
    }
    get nwsapi() {
        if (!this._nwsapi) {
            this._nwsapi = nwsapi({
                document: this,
                DOMException: null,
            });
            this._nwsapi.configure({
                LOGERRORS: false,
                IDS_DUPES: true,
                MIXEDCASE: true,
            });
        }
        return this._nwsapi;
    }
    get documentElement() {
        return super.documentElement;
    }
    get body() {
        return super.body;
    }
    get head() {
        return super.head;
    }
    get implementation() {
        return this;
    }
    get firstElementChild() {
        return this.documentElement;
    }
    appendChild(childNode) {
        return super.appendChild(childNode);
    }
    insertBefore(newChild, refChild) {
        return super.insertBefore(newChild, refChild);
    }
    querySelectorAll(selectors) {
        return this.nwsapi.select(selectors);
    }
    getElementsByTagName(tagName) {
        if (this.documentElement)
            return this.documentElement.getElementsByTagName(tagName);
        return [];
    }
    getElementsByClassName(className) {
        if (this.documentElement)
            return this.documentElement.getElementsByClassName(className);
        return [];
    }
    getElementById(elementId) {
        if (this.documentElement)
            return this.documentElement.getElementById(elementId);
        return null;
    }
    createDocument(_namespace, _qualifiedName, _doctype) {
        return new RRDocument();
    }
    createDocumentType(qualifiedName, publicId, systemId) {
        const documentTypeNode = new RRDocumentType(qualifiedName, publicId, systemId);
        documentTypeNode.ownerDocument = this;
        return documentTypeNode;
    }
    createElement(tagName) {
        const upperTagName = tagName.toUpperCase();
        let element;
        switch (upperTagName) {
            case 'AUDIO':
            case 'VIDEO':
                element = new RRMediaElement(upperTagName);
                break;
            case 'IFRAME':
                element = new RRIFrameElement(upperTagName);
                break;
            case 'IMG':
                element = new RRImageElement(upperTagName);
                break;
            case 'CANVAS':
                element = new RRCanvasElement(upperTagName);
                break;
            case 'STYLE':
                element = new RRStyleElement(upperTagName);
                break;
            default:
                element = new RRElement(upperTagName);
                break;
        }
        element.ownerDocument = this;
        return element;
    }
    createElementNS(_namespaceURI, qualifiedName) {
        return this.createElement(qualifiedName);
    }
    createComment(data) {
        const commentNode = new RRComment(data);
        commentNode.ownerDocument = this;
        return commentNode;
    }
    createCDATASection(data) {
        const sectionNode = new RRCDATASection(data);
        sectionNode.ownerDocument = this;
        return sectionNode;
    }
    createTextNode(data) {
        const textNode = new RRText(data);
        textNode.ownerDocument = this;
        return textNode;
    }
}
class RRDocumentType extends BaseRRDocumentTypeImpl(RRNode) {
}
class RRElement extends BaseRRElementImpl(RRNode) {
    constructor(tagName) {
        super(tagName);
        this._style = new cssstyle.CSSStyleDeclaration();
        const style = this._style;
        Object.defineProperty(this.attributes, 'style', {
            get() {
                return style.cssText;
            },
            set(cssText) {
                style.cssText = cssText;
            },
        });
    }
    get style() {
        return this._style;
    }
    attachShadow(_init) {
        return super.attachShadow(_init);
    }
    appendChild(newChild) {
        return super.appendChild(newChild);
    }
    insertBefore(newChild, refChild) {
        return super.insertBefore(newChild, refChild);
    }
    getAttribute(name) {
        const upperName = name && name.toLowerCase();
        if (upperName in this.attributes)
            return this.attributes[upperName];
        return null;
    }
    setAttribute(name, attribute) {
        this.attributes[name.toLowerCase()] = attribute;
    }
    removeAttribute(name) {
        delete this.attributes[name.toLowerCase()];
    }
    get firstElementChild() {
        for (const child of this.childNodes)
            if (child.RRNodeType === NodeType$1.Element)
                return child;
        return null;
    }
    get nextElementSibling() {
        const parentNode = this.parentNode;
        if (!parentNode)
            return null;
        const siblings = parentNode.childNodes;
        const index = siblings.indexOf(this);
        for (let i = index + 1; i < siblings.length; i++)
            if (siblings[i] instanceof RRElement)
                return siblings[i];
        return null;
    }
    querySelectorAll(selectors) {
        const result = [];
        if (this.ownerDocument !== null) {
            this.ownerDocument.nwsapi.select(selectors, this, (element) => {
                if (element !== this)
                    result.push(element);
            });
        }
        return result;
    }
    getElementById(elementId) {
        if (this.id === elementId)
            return this;
        for (const child of this.childNodes) {
            if (child instanceof RRElement) {
                const result = child.getElementById(elementId);
                if (result !== null)
                    return result;
            }
        }
        return null;
    }
    getElementsByClassName(className) {
        let elements = [];
        const queryClassList = new ClassList(className);
        if (this instanceof RRElement &&
            queryClassList.classes.filter((queriedClassName) => this.classList.classes.some((name) => name === queriedClassName)).length == queryClassList.classes.length)
            elements.push(this);
        for (const child of this.childNodes) {
            if (child instanceof RRElement)
                elements = elements.concat(child.getElementsByClassName(className));
        }
        return elements;
    }
    getElementsByTagName(tagName) {
        let elements = [];
        const normalizedTagName = tagName.toUpperCase();
        if (this instanceof RRElement && this.tagName === normalizedTagName)
            elements.push(this);
        for (const child of this.childNodes) {
            if (child instanceof RRElement)
                elements = elements.concat(child.getElementsByTagName(tagName));
        }
        return elements;
    }
}
class RRImageElement extends RRElement {
}
class RRMediaElement extends BaseRRMediaElementImpl(RRElement) {
}
class RRCanvasElement extends RRElement {
    getContext() {
        return null;
    }
}
class RRStyleElement extends RRElement {
    constructor() {
        super(...arguments);
        this._sheet = null;
    }
    get sheet() {
        if (!this._sheet) {
            let result = '';
            for (const child of this.childNodes)
                if (child.RRNodeType === NodeType$1.Text)
                    result += child.textContent;
            this._sheet = cssom.parse(result);
        }
        return this._sheet;
    }
}
class RRIFrameElement extends RRElement {
    constructor(tagName) {
        super(tagName);
        this.width = '';
        this.height = '';
        this.src = '';
        this.contentDocument = new RRDocument();
        this.contentWindow = new RRWindow();
        const htmlElement = this.contentDocument.createElement('HTML');
        this.contentDocument.appendChild(htmlElement);
        htmlElement.appendChild(this.contentDocument.createElement('HEAD'));
        htmlElement.appendChild(this.contentDocument.createElement('BODY'));
    }
}
class RRText extends BaseRRTextImpl(RRNode) {
    constructor() {
        super(...arguments);
        this.nodeName = '#text';
    }
}
class RRComment extends BaseRRCommentImpl(RRNode) {
    constructor() {
        super(...arguments);
        this.nodeName = '#comment';
    }
}
class RRCDATASection extends BaseRRCDATASectionImpl(RRNode) {
    constructor() {
        super(...arguments);
        this.nodeName = '#cdata-section';
    }
}

function polyfillPerformance() {
    if (typeof window !== 'undefined' || 'performance' in global)
        return;
    global
        .performance = require('perf_hooks').performance;
}
function polyfillRAF() {
    if (typeof window !== 'undefined' || 'requestAnimationFrame' in global)
        return;
    const FPS = 60, INTERVAL = 1000 / FPS;
    let timeoutHandle = null, rafCount = 0, requests = Object.create(null);
    function onFrameTimer() {
        const currentRequests = requests;
        requests = Object.create(null);
        timeoutHandle = null;
        Object.keys(currentRequests).forEach(function (id) {
            const request = currentRequests[id];
            if (request)
                request(Date.now());
        });
    }
    function requestAnimationFrame(callback) {
        const cbHandle = ++rafCount;
        requests[cbHandle] = callback;
        if (timeoutHandle === null)
            timeoutHandle = setTimeout(onFrameTimer, INTERVAL);
        return cbHandle;
    }
    function cancelAnimationFrame(handleId) {
        delete requests[handleId];
        if (Object.keys(requests).length === 0 && timeoutHandle !== null) {
            clearTimeout(timeoutHandle);
            timeoutHandle = null;
        }
    }
    global.requestAnimationFrame = requestAnimationFrame;
    global.cancelAnimationFrame = cancelAnimationFrame;
}
function polyfillEvent() {
    if (typeof Event !== 'undefined')
        return;
    global.Event = function () { };
}
function polyfillNode() {
    if (typeof Node !== 'undefined')
        return;
    global.Node = RRNode;
}
function polyfillDocument() {
    if (typeof document !== 'undefined')
        return;
    const rrdom = new RRDocument();
    (() => {
        rrdom.appendChild(rrdom.createElement('html'));
        rrdom.documentElement.appendChild(rrdom.createElement('head'));
        rrdom.documentElement.appendChild(rrdom.createElement('body'));
    })();
    global.document = rrdom;
}

polyfillPerformance();
polyfillRAF();
polyfillEvent();
polyfillNode();
polyfillDocument();

export { RRCDATASection, RRCanvasElement, RRComment, RRDocument, RRDocumentType, RRElement, RRIFrameElement, RRImageElement, RRMediaElement, RRNode, RRStyleElement, RRText, RRWindow };
